{$mode objfpc}
{$h+}
program runfpc;

uses ctypes, libwasmedge;

var
  ConfCxt : PWasmEdge_ConfigureContext;
  VMCxt : PWasmEdge_VMContext;
  Returns, Params : Array[0..0] of TWasmEdge_Value;
  FuncName : TWasmEdge_String;
  Res : TWasmEdge_Result;
  pmodule : pcchar;
  WasiModule : PWasmEdge_ModuleInstanceContext;
  ModName : TWasmEdge_String;

begin
  Writeln('Loading library...');
  Loadlibwasmedge('./'+libwasmname);
  { Create the configure context and add the WASI support.
    This step is not necessary unless you need WASI support.}
  Writeln('Adding WASI environment...');
    
  ConfCxt:=WasmEdge_ConfigureCreate();
  WasmEdge_ConfigureAddHostRegistration(ConfCxt, WasmEdge_HostRegistration_Wasi);
  Writeln('Creating engine...');
  { The configure and store context to the VM creation can be NULL. }
  VMCxt:=WasmEdge_VMCreate(ConfCxt,Nil);
  WasiModule:=WasmEdge_VMGetImportModuleContext(VMCxt,WasmEdge_HostRegistration_Wasi);
  { Initialize the WASI. }
  WasmEdge_ModuleInstanceInitWASI(WasiModule,Nil,0,Nil,0,Nil,0);
  { Run the WASM function from file. }
  Writeln('Loading webassembly module...');
  ModName:=WasmEdge_StringCreateByCString(Pcchar('prog'));
  pmodule:='hello.wasm';
  Res := WasmEdge_VMRegisterModuleFromFile(VMCxt, modname, pmodule);
  if (WasmEdge_ResultOK(Res)) then
    Writeln('Loaded OK')
  else
    Writeln('Error message: ', WasmEdge_ResultGetMessage(Res));
  Writeln('Running function "_start"');
  { Function name. }
  FuncName:=WasmEdge_StringCreateByCString(Pcchar('_start'));
  Res:=WasmEdge_VMExecuteRegistered(VMCxt, ModName, FuncName,@Params, 0,@Returns,0);
  if (WasmEdge_ResultOK(Res)) then
    Writeln('Run OK')
  else
    Writeln('Error message: ', Pcchar(WasmEdge_ResultGetMessage(Res)));
  Writeln('Exit code: ',WasmEdge_ModuleInstanceWASIGetExitCode(WasiModule));
  Writeln('Cleaning up...');
  { Resources deallocations. }
  WasmEdge_VMDelete(VMCxt);
  WasmEdge_ConfigureDelete(ConfCxt);
  WasmEdge_StringDelete(FuncName);
end.

